/*
 * 2D Game Engine 
 * Render.h: 
 * Based on pikuma.com 2D game engine in C++ and Lua course
 * Copyright (c) 2021 986-Studio. All rights reserved.
 *
 * Created by Manoël Trapier on 12/02/2021.
 */

#ifndef GAMEENGINE_RENDER_H
#define GAMEENGINE_RENDER_H

#include <SDL.h>

#include <AssetStore.h>
#include <ECS.h>
#include <Components/Transform.h>
#include <Components/Sprite.h>

class RenderSystem: public System
{
public:
    RenderSystem()
    {
        this->requireComponent<TransformComponent>();
        this->requireComponent<SpriteComponent>();
    }

    void update(SDL_Renderer *renderer, std::unique_ptr<AssetStore> &assetStore)
    {
        struct RenderableEntity
        {
            TransformComponent transformComponent;
            SpriteComponent spriteComponent;
        };
        std::vector<RenderableEntity> renderableEntities;
        for(auto entity: this->getSystemEntities())
        {
            RenderableEntity renderableEntity;
            renderableEntity.spriteComponent = entity.getComponent<SpriteComponent>();
            renderableEntity.transformComponent = entity.getComponent<TransformComponent>();
            renderableEntities.emplace_back(renderableEntity);
        }

        std::sort(renderableEntities.begin(), renderableEntities.end(), [](const RenderableEntity &a, const RenderableEntity &b)
        {
           return a.spriteComponent.zIndex < b.spriteComponent.zIndex;
        });

        for(auto entity: renderableEntities)
        {
            const auto transform = entity.transformComponent;
            const auto sprite = entity.spriteComponent;

            SDL_Texture *texture = assetStore->getTexture(sprite.assetId);
            SDL_Rect destRect =
            {
                static_cast<int>(transform.position.x),
                static_cast<int>(transform.position.y),
                static_cast<int>(sprite.width * transform.scale.x),
                static_cast<int>(sprite.height * transform.scale.y)
            };

            SDL_RenderCopyEx(renderer, texture, &sprite.sourceRect, &destRect,
                             transform.rotation, NULL, SDL_FLIP_NONE);

        }
    }
};

#endif /* GAMEENGINE_RENDER_H */
